@@ -21,10 +21,35 @@ use std::{
     cmp::Ordering,
     fmt::{Debug, Write},
     io::{self, ErrorKind, Read},
-    ops::{Bound, Deref, RangeBounds},
+    ops::{Bound, Deref, DerefMut, RangeBounds},
     sync::Arc,
 };
 
+/// Formats the bytes contained in `slice` into the provided `writer`. This
+/// function groups the printed output in chukns of 8 hexadecimal characters,
+/// and inserts spaces between each group.
+///
+/// This function powers the [`Debug`] trait implementations within the crate.
+///
+/// ```rust
+/// let mut printed = String::default();
+/// arc_bytes::print_bytes(b"\x01\x23\x45\x67\x89", &mut printed).unwrap();
+/// assert_eq!(printed, "01234567 89");
+/// ```
+pub fn print_bytes<W: Write>(mut slice: &[u8], writer: &mut W) -> std::fmt::Result {
+    while !slice.is_empty() {
+        let (chunk, remaining) = slice.split_at(4.min(slice.len()));
+        slice = remaining;
+        for byte in chunk {
+            write!(writer, "{:02x}", byte)?;
+        }
+        if !slice.is_empty() {
+            writer.write_char(' ')?;
+        }
+    }
+    Ok(())
+}
+
 /// An immutable buffer of bytes that can be cloned, sliced, and read into
 /// multiple parts using a single refernce to the underlying buffer.
 ///
@@ -75,18 +100,9 @@ fn default_is_new() {
 
 impl<'a> Debug for ArcBytes<'a> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let mut slice = self.as_slice();
+        let slice = self.as_slice();
         write!(f, "ArcBytes {{ length: {}, bytes: [", slice.len())?;
-        while !slice.is_empty() {
-            let (chunk, remaining) = slice.split_at(4.min(slice.len()));
-            slice = remaining;
-            for byte in chunk {
-                write!(f, "{:02x}", byte)?;
-            }
-            if !slice.is_empty() {
-                f.write_char(' ')?;
-            }
-        }
+        print_bytes(slice, f)?;
         f.write_str("] }")
     }
 }
@@ -114,6 +130,12 @@ impl<'a> PartialEq<[u8]> for ArcBytes<'a> {
     }
 }
 
+impl<'a, const SIZE: usize> PartialEq<[u8; SIZE]> for ArcBytes<'a> {
+    fn eq(&self, other: &[u8; SIZE]) -> bool {
+        self.as_slice().cmp(other) == Ordering::Equal
+    }
+}
+
 impl<'a, 'b> PartialEq<&'b [u8]> for ArcBytes<'a> {
     fn eq(&self, other: &&'b [u8]) -> bool {
         self.as_slice().cmp(other) == Ordering::Equal
@@ -126,6 +148,12 @@ impl<'a> PartialOrd<[u8]> for ArcBytes<'a> {
     }
 }
 
+impl<'a, 'b, const SIZE: usize> PartialOrd<&'b [u8; SIZE]> for ArcBytes<'a> {
+    fn partial_cmp(&self, other: &&'b [u8; SIZE]) -> Option<Ordering> {
+        self.as_slice().partial_cmp(&other[..])
+    }
+}
+
 impl<'b, 'a> PartialOrd<&'b [u8]> for ArcBytes<'a> {
     fn partial_cmp(&self, other: &&'b [u8]) -> Option<Ordering> {
         self.as_slice().partial_cmp(other)
@@ -466,6 +494,12 @@ impl<'a> Deref for ArcBytes<'a> {
     }
 }
 
+impl<'a> std::borrow::Borrow<[u8]> for ArcBytes<'a> {
+    fn borrow(&self) -> &[u8] {
+        &**self
+    }
+}
+
 impl<'a> Read for ArcBytes<'a> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         let end = self.buffer.len().min(self.position + buf.len());
@@ -538,6 +572,90 @@ fn iterator_tests() {
     assert_eq!(iterated, vec![0, 1, 2]);
 }
 
+/// An instance of [`ArcBytes`] that is not borrowing its underlying data.
+#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+pub struct OwnedBytes(pub ArcBytes<'static>);
+
+impl Deref for OwnedBytes {
+    type Target = ArcBytes<'static>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl DerefMut for OwnedBytes {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.0
+    }
+}
+
+impl From<Vec<u8>> for OwnedBytes {
+    fn from(vec: Vec<u8>) -> Self {
+        Self(ArcBytes::from(vec))
+    }
+}
+
+impl<'a> From<ArcBytes<'a>> for OwnedBytes {
+    fn from(bytes: ArcBytes<'a>) -> Self {
+        Self(bytes.into_owned())
+    }
+}
+
+impl<'a> From<&'a [u8]> for OwnedBytes {
+    fn from(bytes: &'a [u8]) -> Self {
+        Self(ArcBytes::owned(bytes.to_vec()))
+    }
+}
+
+impl<'a, const SIZE: usize> From<&'a [u8; SIZE]> for OwnedBytes {
+    fn from(bytes: &'a [u8; SIZE]) -> Self {
+        Self(ArcBytes::owned(bytes.to_vec()))
+    }
+}
+
+impl std::borrow::Borrow<[u8]> for OwnedBytes {
+    fn borrow(&self) -> &[u8] {
+        &**self
+    }
+}
+
+impl PartialEq<[u8]> for OwnedBytes {
+    fn eq(&self, other: &[u8]) -> bool {
+        self.0 == other
+    }
+}
+
+impl PartialOrd<[u8]> for OwnedBytes {
+    fn partial_cmp(&self, other: &[u8]) -> Option<Ordering> {
+        self.0.partial_cmp(other)
+    }
+}
+
+impl<'a> PartialEq<&'a [u8]> for OwnedBytes {
+    fn eq(&self, other: &&'a [u8]) -> bool {
+        self.0 == *other
+    }
+}
+
+impl<'a> PartialOrd<&'a [u8]> for OwnedBytes {
+    fn partial_cmp(&self, other: &&'a [u8]) -> Option<Ordering> {
+        self.0.partial_cmp(other)
+    }
+}
+
+impl<'a, const SIZE: usize> PartialEq<&'a [u8; SIZE]> for OwnedBytes {
+    fn eq(&self, other: &&'a [u8; SIZE]) -> bool {
+        self.0 == *other
+    }
+}
+
+impl<'a, const SIZE: usize> PartialOrd<&'a [u8; SIZE]> for OwnedBytes {
+    fn partial_cmp(&self, other: &&'a [u8; SIZE]) -> Option<Ordering> {
+        self.0.partial_cmp(other)
+    }
+}
+
 /// Efficient serialization implementation, ensuring bytes are written as a
 /// buffer of bytes not as a sequence.
 #[cfg(feature = "serde")]
