@@ -9,7 +9,7 @@ use serde::{
     Deserialize, Serialize,
 };
 
-use crate::ArcBytes;
+use crate::{print_bytes, ArcBytes, OwnedBytes};
 
 impl<'a> Serialize for ArcBytes<'a> {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
@@ -31,8 +31,31 @@ impl<'a, 'de: 'a> Deserialize<'de> for ArcBytes<'a> {
     }
 }
 
+impl Serialize for OwnedBytes {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: serde::Serializer,
+    {
+        serializer.serialize_bytes(self.0.as_slice())
+    }
+}
+
+impl<'de> Deserialize<'de> for OwnedBytes {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: serde::Deserializer<'de>,
+    {
+        deserializer
+            .deserialize_byte_buf(BufferVisitor)
+            .map(|bytes| match bytes {
+                Cow::Borrowed(borrowed) => Self(ArcBytes::owned(borrowed.to_vec())),
+                Cow::Owned(vec) => Self(ArcBytes::owned(vec)),
+            })
+    }
+}
+
 /// A `Vec<u8>` wrapper that supports serializing efficiently in Serde.
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
+#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
 pub struct Bytes(pub Vec<u8>);
 
 impl Bytes {
@@ -44,6 +67,15 @@ impl Bytes {
     }
 }
 
+impl<'a> std::fmt::Debug for Bytes {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let slice = self.as_slice();
+        write!(f, "Bytes {{ length: {}, bytes: [", slice.len())?;
+        print_bytes(slice, f)?;
+        f.write_str("] }")
+    }
+}
+
 impl From<Vec<u8>> for Bytes {
     fn from(buffer: Vec<u8>) -> Self {
         Self(buffer)
@@ -106,7 +138,7 @@ impl<'de> Deserialize<'de> for Bytes {
 }
 
 /// A `Cow<'a, [u8]>` wrapper that supports serializing efficiently in Serde.
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
+#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
 pub struct CowBytes<'a>(pub Cow<'a, [u8]>);
 
 impl<'a> CowBytes<'a> {
@@ -128,6 +160,24 @@ impl<'a> CowBytes<'a> {
     }
 }
 
+impl<'a> std::fmt::Debug for CowBytes<'a> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let slice = &self[..];
+        write!(
+            f,
+            "CowBytes {{ length: {}, bytes: {}[",
+            slice.len(),
+            if matches!(self.0, Cow::Borrowed(_)) {
+                "&"
+            } else {
+                ""
+            }
+        )?;
+        print_bytes(slice, f)?;
+        f.write_str("] }")
+    }
+}
+
 impl<'a> From<Bytes> for CowBytes<'a> {
     fn from(bytes: Bytes) -> Self {
         CowBytes(Cow::Owned(bytes.0))
